import 'enum.dart';

/// Each time you start the SDK, you will need to get a new SDK token for the user and initialize the SDK with the token and a few other parameters.
///
class PhylloConfig {
  /// The environment to connect to
  final PhylloEnvironment environment;

  /// The name to be displayed to the user while obtaining consent
  final String clientDisplayName;

  /// The user Id generated by Phyllo using API (see `https://docs.getphyllo.com/docs/api-reference/b3A6MTQwNjEzNzY-create-a-user`)
  final String userId;

  /// The SDK token generated
  final String token;

  /// (Optional) The work platform ID to connect with, if you want the user to skip the platform selection screen and taken directly to platform specific login screen
  final String? workPlatformId;

  PhylloConfig({
    required this.environment,
    required this.clientDisplayName,
    required this.userId,
    required this.token,
    this.workPlatformId,
  });

  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'clientDisplayName': clientDisplayName,
      'userId': userId,
      'token': token,
      'workPlatformId': workPlatformId,
      'environment': environment.name,
    };
  }

  @override
  String toString() {
    return 'PhylloConfig(environment: $environment, clientDisplayName: $clientDisplayName, userId: $userId, token: $token, workPlatformId: $workPlatformId)';
  }
}
